package icu.fur93.esp32_car

import android.Manifest
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothGatt
import android.bluetooth.BluetoothGattCallback
import android.bluetooth.BluetoothGattCharacteristic
import android.bluetooth.le.BluetoothLeScanner
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanFilter
import android.bluetooth.le.ScanResult
import android.bluetooth.le.ScanSettings
import android.content.pm.ActivityInfo
import android.content.pm.PackageManager
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.os.ParcelUuid
import android.util.Log
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.compose.foundation.gestures.Orientation
import androidx.compose.foundation.layout.*
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.items
import androidx.compose.material3.*
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.graphicsLayer
import androidx.compose.ui.unit.dp
import androidx.core.app.ActivityCompat
import java.util.UUID
import kotlin.reflect.KFunction1

class MainActivity : ComponentActivity() {
    private val serviceUUID = "6E400001-B5A3-F393-E0A9-E50E24DCCA9E"
    private val rxCharUUID = "6E400002-B5A3-F393-E0A9-E50E24DCCA9E"
    private val txCharUUID = "6E400003-B5A3-F393-E0A9-E50E24DCCA9E"

    private var bluetoothGatt: BluetoothGatt? = null
    private var rxCharacteristic: BluetoothGattCharacteristic? = null
    private var bleScanner: BluetoothLeScanner? = null

    // 将deviceList移动到Activity作用域内并使用MutableList
    private val deviceList = mutableListOf<ScanResult>()

    // 添加可观察的连接状态
    private val _isConnected = mutableStateOf(false)
    val isConnected: State<Boolean> = _isConnected

    private val scanCallback = object : ScanCallback() {
        override fun onScanResult(callbackType: Int, result: ScanResult) {
            super.onScanResult(callbackType, result)
            Log.d("MainActivity", "onScanResult: ${result.device.name} - ${result.device.address}")
            if (!deviceList.any { it.device.address == result.device.address }) {
                deviceList.add(result)
            }
        }

        override fun onScanFailed(errorCode: Int) {
            super.onScanFailed(errorCode)
            Log.e("MainActivity", "扫描失败，错误码: $errorCode")
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        
        // 设置横屏
        requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE

        // 检查并请求所有必要的权限
        val requiredPermissions = arrayOf(
            Manifest.permission.BLUETOOTH_SCAN,
            Manifest.permission.BLUETOOTH_CONNECT,
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.ACCESS_COARSE_LOCATION
        )

        val missingPermissions = requiredPermissions.filter {
            ActivityCompat.checkSelfPermission(this, it) != PackageManager.PERMISSION_GRANTED
        }

        if (missingPermissions.isNotEmpty()) {
            ActivityCompat.requestPermissions(
                this,
                missingPermissions.toTypedArray(),
                BLUETOOTH_PERMISSION_REQUEST_CODE
            )
        } else {
            // 如果已经有所有权限，直接开始扫描
            val bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
            bleScanner = bluetoothAdapter?.bluetoothLeScanner
            startBleScan()
        }

        setContent {
            GamepadScreen(
                deviceList = deviceList,
                isConnected = isConnected.value,
                onStartScan = ::startBleScan,
                onConnect = ::connectToDevice,
                onSendCommand = ::sendCommand
            )
        }
    }

    private fun sendCommand(command: ByteArray) {
        rxCharacteristic?.let { characteristic ->
            characteristic.value = command;
            if (ActivityCompat.checkSelfPermission(
                    this,
                    Manifest.permission.BLUETOOTH_CONNECT
                ) != PackageManager.PERMISSION_GRANTED
            ) {
                // 请求蓝牙连接权限
                ActivityCompat.requestPermissions(
                    this,
                    arrayOf(Manifest.permission.BLUETOOTH_CONNECT),
                    BLUETOOTH_PERMISSION_REQUEST_CODE
                )
                return
            }
            bluetoothGatt?.writeCharacteristic(characteristic)
        }
    }

    private fun connectToDevice(result: ScanResult) {
        if (ActivityCompat.checkSelfPermission(
                this,
                Manifest.permission.BLUETOOTH_CONNECT
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                this,
                arrayOf(Manifest.permission.BLUETOOTH_CONNECT),
                BLUETOOTH_PERMISSION_REQUEST_CODE
            )
            return
        }

        result.device.connectGatt(this, false, object : BluetoothGattCallback() {
            override fun onConnectionStateChange(gatt: BluetoothGatt?, status: Int, newState: Int) {
                super.onConnectionStateChange(gatt, status, newState)
                when (newState) {
                    BluetoothGatt.STATE_CONNECTED -> {
                        bluetoothGatt = gatt
                        _isConnected.value = true
                        gatt?.discoverServices()
                    }

                    BluetoothGatt.STATE_DISCONNECTED -> {
                        _isConnected.value = false
                        bluetoothGatt = null
                    }
                }
            }

            override fun onServicesDiscovered(gatt: BluetoothGatt?, status: Int) {
                super.onServicesDiscovered(gatt, status)
                if (status == BluetoothGatt.GATT_SUCCESS) {
                    gatt?.getService(UUID.fromString(serviceUUID))?.let { service ->
                        rxCharacteristic = service.getCharacteristic(UUID.fromString(rxCharUUID))
                    }
                }
            }
        })
    }

    private fun startBleScan() {
        if (ActivityCompat.checkSelfPermission(
                this,
                Manifest.permission.BLUETOOTH_SCAN
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                this,
                arrayOf(
                    Manifest.permission.BLUETOOTH_SCAN,
                    Manifest.permission.ACCESS_FINE_LOCATION
                ),
                BLUETOOTH_PERMISSION_REQUEST_CODE
            )
            return
        }

        // 先停止当前扫描
        try {
            bleScanner?.stopScan(scanCallback)
        } catch (e: Exception) {
            Log.e("MainActivity", "停止扫描失败: ${e.message}")
        }

        // 清空之前的扫描结果
        deviceList.clear()

        Log.d("MainActivity", "开始扫描...")

        // 添加UUID过滤器
        val scanFilter = ScanFilter.Builder()
            .setServiceUuid(ParcelUuid.fromString(serviceUUID))
            .build()

        val scanSettings = ScanSettings.Builder()
            .setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY)
            .build()

        try {
//            bleScanner?.startScan(listOf(scanFilter), scanSettings, scanCallback)
            bleScanner?.startScan(scanCallback)
        } catch (e: Exception) {
            Log.e("MainActivity", "开始扫描失败: ${e.message}")
        }

        // 10秒后停止扫描
        Handler(Looper.getMainLooper()).postDelayed({
            stopBleScan()
            Log.d("MainActivity", "停止扫描，发现 ${deviceList.size} 个设备")
        }, 10000)
    }

    private fun stopBleScan() {
        if (ActivityCompat.checkSelfPermission(
                this,
                Manifest.permission.BLUETOOTH_SCAN
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            return
        }
        try {
            bleScanner?.stopScan(scanCallback)
        } catch (e: Exception) {
            Log.e("MainActivity", "停止扫描失败: ${e.message}")
        }
    }

    // 添加权限请求结果处理
    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == BLUETOOTH_PERMISSION_REQUEST_CODE) {
            if (grantResults.all { it == PackageManager.PERMISSION_GRANTED }) {
                Log.d("MainActivity", "所有权限已获得")
                // 权限获得后重新初始化蓝牙
                val bluetoothAdapter = BluetoothAdapter.getDefaultAdapter()
                bleScanner = bluetoothAdapter?.bluetoothLeScanner
            } else {
                Log.e("MainActivity", "部分权限未获得")
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        // 确保在Activity销毁时停止扫描
        stopBleScan()
    }

    companion object {
        private const val BLUETOOTH_PERMISSION_REQUEST_CODE = 1
    }
}

@Composable
fun GamepadScreen(
    deviceList: List<ScanResult>,
    isConnected: Boolean,
    onStartScan: () -> Unit,
    onConnect: (ScanResult) -> Unit,
    onSendCommand: KFunction1<ByteArray, Unit>
) {
    var showDeviceList by remember { mutableStateOf(false) }
    var sliderValue by remember { mutableStateOf(0f) }

    Box(modifier = Modifier.fillMaxSize()) {
        // 设备选择按钮
        Button(
            onClick = { showDeviceList = true },
            modifier = Modifier
                .align(Alignment.TopEnd)
                .padding(16.dp)
        ) {
            Text(if (isConnected) "已连接" else "选择设备")
        }

        // 方向控制布局
        Row(
            modifier = Modifier.fillMaxSize(),
            horizontalArrangement = Arrangement.SpaceEvenly,
            verticalAlignment = Alignment.CenterVertically
        ) {
            // 左侧控制区
            Column(
                horizontalAlignment = Alignment.CenterHorizontally,
                verticalArrangement = Arrangement.Center,
                modifier = Modifier.weight(1f)
            ) {
                Row(
                    horizontalArrangement = Arrangement.spacedBy(16.dp),
                    modifier = Modifier.padding(vertical = 16.dp)
                ) {
                    Button(onClick = {
                        onSendCommand(
                            byteArrayOf(
                                0x00, 0x06, 0x21, 0x00, 0x10, 0xff.toByte(),
                                0xff.toByte()
                            )
                        )
                    }) { Text("左转") }
                    Button(onClick = {
                        onSendCommand(
                            byteArrayOf(
                                0x00, 0x06, 0x20, 0x01, 0x10, 0xff.toByte(),
                                0xff.toByte()
                            )
                        )
                    }) { Text("右转") }
                }
            }

            // 右侧控制区
            Column(
                horizontalAlignment = Alignment.CenterHorizontally,
                verticalArrangement = Arrangement.Center,
                modifier = Modifier.weight(1f)
            ) {
                // 垂直滑杆
                Slider(
                    value = sliderValue,
                    onValueChange = { newValue ->
                        sliderValue = newValue
                        val speed = (newValue * 255).toInt()
                        val command = if (speed >= 0) {
                            byteArrayOf(
                                0x00, 0x06, 0x20, 0x01, speed.toByte(),
                                0xff.toByte()
                            )
                        } else {
                            byteArrayOf(
                                0x00, 0x06, 0x20, 0x02, (-speed).toByte(),
                                0xff.toByte()
                            )
                        }
                        onSendCommand(command)
                    },
                    valueRange = -1f..1f,
                    modifier = Modifier
                        .height(20.dp)
                        .width(200.dp)
                        .padding(vertical = 16.dp)
                        .graphicsLayer(rotationZ = 270f) // 旋转 270 度
                )
            }
            
        }

        // 设备列表对话框
        if (showDeviceList) {
            AlertDialog(
                onDismissRequest = { showDeviceList = false },
                title = { Text("选择设备") },
                text = {
                    // 设备列表
                    LazyColumn {
                        items(deviceList) { device ->
                            TextButton(
                                onClick = {
                                    onConnect(device)
                                    showDeviceList = false
                                },
                                modifier = Modifier.fillMaxWidth()
                            ) {
                                Column(
                                    modifier = Modifier.fillMaxWidth(),
                                    horizontalAlignment = Alignment.Start
                                ) {
                                    Text(device.device.name ?: "未知设备")
                                    Text(
                                        device.device.address,
                                        style = MaterialTheme.typography.bodySmall
                                    )
                                }
                            }
                        }
                    }
                },
                dismissButton = {
                    TextButton(
                        onClick = { onStartScan() }
                    ) {
                        Text("扫描设备")
                    }
                },
                confirmButton = {
                    TextButton(onClick = { showDeviceList = false }) {
                        Text("取消")
                    }
                }
            )
        }
    }
}